@@ -10,8 +10,8 @@ migrate-ebs-image.pl     Copy an EBS-backed Amazon Image from one region to anot
 
 =head1 DESCRIPTION
 
-This script copies an EBS-backed Unix/Linux AMI located in the EC2
-region indicated by --from to the region indicated by --to. All
+This script copies an EBS-backed Unix/Linux/windows AMI located in the
+EC2 region indicated by --from to the region indicated by --to. All
 associated volume snapshots, including LVM and RAID volumes, are
 migrated as well.
 
@@ -19,9 +19,12 @@ If --from is omitted, then the source region is derived from the
 endpoint URL contained in the EC2_URL environment variable. The --to
 option is required.
 
-This script will ONLY work with EBS backed Linux images. It does not
-work with Windows images due to fundamental limitations of the EC2
-API.
+This script works with any EBS backed image, including Linux PVM,
+Linux HVM (cluster) and Windows images. It will B<not> work with any
+instance-store backed image. To migrate such instances, please see one
+of the recipes listed online, for example:
+
+ http://www.dowdandassociates.com/content/howto-move-ec2-instance-store-ami-one-region-another
 
 =head1 COMMAND-LINE OPTIONS
 
@@ -32,13 +35,13 @@ Options can be abbreviated.  For example, you can use -l for
       --to           Region to which the AMI is to be copied (e.g. "us-west-1") REQUIRED
       --access_key   EC2 access key
       --secret_key   EC2 secret key
-      --block-device-mapping 
+      --block_device_mapping 
                      Add additional block devices to the image.
       --endpoint     EC2 URL (defaults to http://ec2.amazonaws.com/)
       --kernel       Force assignment of kernel in destination image.
       --ramdisk      Force assignment of ramdisk in destination image.
       --quiet        Quench status messages
-      --list-regions List the EC2 regions
+      --list_regions List the EC2 regions
 
 The --block-device-mapping (-b) option is used to add ephemeral
 storage to the destination image. Amazon's API doesn't describe
@@ -122,12 +125,12 @@ my $Program_name = basename($0);
 
 GetOptions('from=s'        => \$From,
 	   'to=s'          => \$To,
-	   'access_key=s'  => \$Access_key,
-	   'secret_key=s'  => \$Secret_key,
+	   'access_key|access-key=s'  => \$Access_key,
+	   'secret_key|secret-key=s'  => \$Secret_key,
 	   'endpoint=s'    => \$Endpoint,
 	   'quiet'         => \$Quiet,
-	   'list_regions'  => \$List,
-	   'block-device-mapping=s' => \@Block_devices,
+	   'list_regions|list-regions'  => \$List,
+	   'block_device_mapping|block-device-mapping=s' => \@Block_devices,
 	   'kernel'        => \$Kernel,
 	   'ramdisk'       => \$Ramdisk,
     ) or exec 'perldoc',$0;
